import tensorflow as tf


def init_matrix(m, n):
    initializer = tf.keras.initializers.glorot_normal(42)
    return tf.Variable(initializer(shape=(m, n)), trainable=True)


def init_vector(m):
    initializer = tf.keras.initializers.glorot_normal(42)
    return tf.Variable(initializer(shape=(m, 1)), trainable=True)



import tensorflow as tf

from .utils import init_vector, init_matrix


class Model(tf.keras.Model):
    def __init__(self, p, q, family, regularizers):
        super(Model, self).__init__(p, q, family)

        self.alpha = init_vector(q)
        self.beta = init_matrix(p, q)
        self.family = family

        self.regularizers = regularizers or []

    def call(self, x):
        # forward pass
        eta = self._linear_predictor(self.alpha, self.beta, x)

        # apply regularization terms
        for param, reg in self.regularizers:
            self.add_loss(param * reg(self.alpha, self.beta, self.family))

        return self.family["linkinv"](eta)

    def _linear_predictor(self, alpha, beta, x):
        eta = tf.matmul(x, beta) + alpha
        return eta



import tensorflow as tf

from .model import Model
from .families import gaussian_family


class RegReg:
    def __init__(self, x, y, family=gaussian_family, regularizers=None):
        self.x = x
        self.y = y

        self.model = Model(
            self.x.shape[1], self.y.shape[1], family, regularizers=regularizers
        )
        self.model.compile(
            optimizer="adam",
            loss=lambda y_true, y_pred: self._loss_wrapper(
                y_true, y_pred, family["loss"]
            ),
        )

    def fit(self, epochs=100000):
        es = tf.keras.callbacks.EarlyStopping(
            monitor="loss", mode="min", verbose=1, patience=50
        )
        return self.model.fit(self.x, self.y, epochs=epochs, callbacks=[es])

    def _loss_wrapper(self, y_true, y_pred, loss_func):
        return loss_func(y_true, y_pred)

    @property
    def intercept_(self):
        return self.model.alpha.numpy()

    @property
    def coef_(self):
        return self.model.beta.numpy()



import tensorflow as tf


def ridge(alpha, beta, family):
    return tf.reduce_sum(tf.square(beta))


def lasso(alpha, beta, family):
    return tf.reduce_sum(tf.abs(beta))


def network_fusion_x(graph):
    graph = tf.cast(graph, tf.float32)

    def tmp(alpha, beta, family):
        return tf.linalg.trace(tf.matmul(tf.transpose(beta), tf.matmul(graph, beta)))

    return tmp


def network_fusion_y(graph):
    graph = tf.cast(graph, tf.float32)

    def tmp(alpha, beta, family):
        return tf.linalg.trace(tf.matmul(beta, tf.matmul(graph, tf.transpose(beta))))

    return tmp



import tensorflow as tf
import tensorflow_probability as tfp


def gaussian_loss(y_true, y_pred):
    return tf.reduce_mean(tf.square(y_true - y_pred))


gaussian_family = {
    "link": lambda x: x,
    "linkinv": lambda x: x,
    "loss": gaussian_loss,
}


def binomial_loss(y_true, y_pred):
    obj = 0
    for j in range(y_true.shape[1]):
        prob = tfp.distributions.Bernoulli(probs=y_pred[:, j], validate_args=True)
        obj += tf.reduce_sum(prob.log_prob(y_true[:, j]))
    return -obj


binomial_family = {
    "link": lambda p: tf.math.log(p / (1 - p)),  # logit
    "linkinv": lambda x: 1 / (1 + tf.exp(-x)),  # logistic
    "loss": binomial_loss,
}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['reggy']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.3.5,<2.0.0',
 'tensorflow-probability>=0.15.0,<0.16.0',
 'tensorflow>=2.7.0,<3.0.0']

setup_kwargs = {
    'name': 'reggy',
    'version': '0.2.0',
    'description': 'Python package for regularized regressions.',
    'long_description': '# reggy\n\n[![PyPI](https://img.shields.io/pypi/v/reggy.svg?style=flat)](https://pypi.python.org/pypi/reggy)\n[![Tests](https://github.com/kpj/reggy/workflows/Tests/badge.svg)](https://github.com/kpj/reggy/actions)\n\nPython package for regularized regressions.\n\nSupported regularization terms:\n* Ridge\n* LASSO\n* Network-fusion penalty (https://www.ncbi.nlm.nih.gov/pmc/articles/PMC6030897/)\n\n\n## Installation\n\n```bash\n$ pip install reggy\n```\n\n\n## Usage\n\nA simple example with LASSO regularization:\n```python\nimport reggy\nimport numpy as np\n\n\nalpha = 0.3\nbeta = 1.7\n\nX = np.random.normal(size=(100, 1))\ny = np.random.normal(X * beta + alpha, size=(100, 1))\n\nmodel = reggy.RegReg(X, y, family=reggy.gaussian_family, regularizers=[(0.5, reggy.lasso)])\nmodel.fit()\n\nprint(model.intercept_, model.coef_)\n## [[0.22491232]] [[0.9219889]]\n```\n',
    'author': 'kpj',
    'author_email': 'kim.philipp.jablonski@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kpj/reggy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9.0,<3.10.0',
}


setup(**setup_kwargs)
